/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                       Copyright (C) A. P. H.                           **
**                           BIALYSTOK 2019                               **
**                                                                        **
**                        MODULE UART1 example                            **
**                                                                        **
**                     Mikrokontroler ARM: LPC2366                        **
**                                                                        **
**                     Wersja z dnia : 2019.01.20                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart1.h"

#define IRQCallCntLimit                 500

static ULONG IRQCallCounter ;

static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
} /* SoftwareInit */


static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART1InitEnvir ( ) ;
} /* HardwareInit */


UCHAR Hello1 [ ]      = "\r\nWitaj koles przy kompie\r\n\r\nNapisz cos\r\n" ;
UCHAR Hello2 [ ]      = "\r\nNapisales: " ;


static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART1SendString ( Hello1 ) ;
} /* HelloMessage */


static void EncodeControlChar ( UCHAR Ch )
{
  /*-------------------------------------------------------------------------*/
  switch ( Ch )
  {
    case 0x00 :
      UART1SendString ( ( UCHAR * ) "null" ) ;
      break ;
    case 0x01 :
      UART1SendString ( ( UCHAR * ) "CTRL A" ) ;
      break ;
    case 0x02 :
      UART1SendString ( ( UCHAR * ) "CTRL B" ) ;
      break ;
    case 0x03 :
      UART1SendString ( ( UCHAR * ) "CTRL C" ) ;
      break ;
    case 0x04 :
      UART1SendString ( ( UCHAR * ) "CTRL D" ) ;
      break ;
    case 0x05 :
      UART1SendString ( ( UCHAR * ) "CTRL E" ) ;
      break ;
    case 0x06 :
      UART1SendString ( ( UCHAR * ) "CTRL F" ) ;
      break ;
    case 0x07 :
      UART1SendString ( ( UCHAR * ) "CTRL G" ) ;
      break ;
    case 0x08 :
      UART1SendString ( ( UCHAR * ) "CTRL H" ) ;
      break ;
    case 0x09 :
      UART1SendString ( ( UCHAR * ) "CTRL I" ) ;
      break ;
    case 0x0A :
      UART1SendString ( ( UCHAR * ) "CTRL J" ) ;
      break ;
    case 0x0B :
      UART1SendString ( ( UCHAR * ) "CTRL K" ) ;
      break ;
    case 0x0C :
      UART1SendString ( ( UCHAR * ) "CTRL L" ) ;
      break ;
    case 0x0D :
      UART1SendString ( ( UCHAR * ) "CTRL M" ) ;
      break ;
    case 0x0E :
      UART1SendString ( ( UCHAR * ) "CTRL N" ) ;
      break ;
    case 0x0F :
      UART1SendString ( ( UCHAR * ) "CTRL O" ) ;
      break ;
    case 0x10 :
      UART1SendString ( ( UCHAR * ) "CTRL P" ) ;
      break ;
    case 0x11 :
      UART1SendString ( ( UCHAR * ) "CTRL Q" ) ;
      break ;
    case 0x12 :
      UART1SendString ( ( UCHAR * ) "CTRL R" ) ;
      break ;
    case 0x13 :
      UART1SendString ( ( UCHAR * ) "CTRL S" ) ;
      break ;
    case 0x14 :
      UART1SendString ( ( UCHAR * ) "CTRL T" ) ;
      break ;
    case 0x15 :
      UART1SendString ( ( UCHAR * ) "CTRL U" ) ;
      break ;
    case 0x16 :
      UART1SendString ( ( UCHAR * ) "CTRL V" ) ;
      break ;
    case 0x17 :
      UART1SendString ( ( UCHAR * ) "CTRL W" ) ;
      break ;
    case 0x18 :
      UART1SendString ( ( UCHAR * ) "CTRL X" ) ;
      break ;
    case 0x19 :
      UART1SendString ( ( UCHAR * ) "CTRL Y" ) ;
      break ;
    case 0x1A :
      UART1SendString ( ( UCHAR * ) "CTRL Z" ) ;
      break ;
    case 0x1B :
      UART1SendString ( ( UCHAR * ) "esc" ) ;
      break ;
    case 0x1C :
      UART1SendString ( ( UCHAR * ) "1C hex" ) ;
      break ;
    case 0x1D :
      UART1SendString ( ( UCHAR * ) "1D hex" ) ;
      break ;
    case 0x1E :
      UART1SendString ( ( UCHAR * ) "1E hex" ) ;
      break ;
    case 0x1F :
      UART1SendString ( ( UCHAR * ) "1F hex" ) ;
      break ;
  } /* switch */ ;
} /* EncodeControlChar */


static UCHAR Prompt [ ]        = "Przyszlo: " ;
static UCHAR ControlChar [ ]   = "<znak kontrolny> " ;

void ProcessChar ( UCHAR Ch )
{
  /*-------------------------------------------------------------------------*/
  UART1SendString ( Prompt ) ;
  if ( Ch < ' ' )
  {
    UART1SendString ( ControlChar ) ;
    EncodeControlChar ( Ch ) ;
  } /* if ... */
  else
  {
    UART1Send ( Ch ) ;
  } /* if ... else */ ;
  UART1Send ( 0x0D ) ;
  UART1Send ( 0x0A ) ;
} /* ProcessChar */


int main ( void )
{
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART1HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  for ( ; ; )
  {
    if ( UART1DataPresent ( ) )
      ProcessChar ( UART1GetData ( ) ) ;
  } /* loop */ ;
} /* main */



