/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                       Copyright (C) A. P. H.                           **
**                           BIALYSTOK 2019                               **
**                                                                        **
**                        MODULE UART1 example                            **
**                                                                        **
**                     Mikrokontroler ARM: LPC2366                        **
**                                                                        **
**                     Wersja z dnia : 2019.01.20                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart1.h"

#define IRQCallCntLimit                 500

static ULONG IRQCallCounter ;
static UCHAR LastDCDModemLine ;
static UCHAR LastRIModemLine ;
static UCHAR LastDSRModemLine ;
static UCHAR LastCTSModemLine ;

static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  LastDCDModemLine = 0 ;
  LastRIModemLine = 0 ;
  LastDSRModemLine = 0 ;
  LastCTSModemLine = 0 ;
} /* SoftwareInit */


static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART1InitEnvir ( ) ;
} /* HardwareInit */


UCHAR Hello1 [ ]      = "\r\nWitaj koles przy kompie\r\n\r\n" ;

static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART1SendString ( Hello1 ) ;
} /* HelloMessage */


void ProcessChar ( UCHAR Ch )
{
  /*-------------------------------------------------------------------------*/
  UART1Send ( Ch ) ;
  UART1Send ( 0x0D ) ;
  UART1Send ( 0x0A ) ;
} /* ProcessChar */


static UCHAR Message01 [ ]   = "Wlaczono linie DCD.\r\n" ;
static UCHAR Message02 [ ]   = "Wylaczono linie DCD.\r\n" ;
static UCHAR Message03 [ ]   = "Wlaczono linie RI.\r\n" ;
static UCHAR Message04 [ ]   = "Wylaczono linie RI.\r\n" ;
static UCHAR Message05 [ ]   = "Wlaczono linie DSR.\r\n" ;
static UCHAR Message06 [ ]   = "Wylaczono linie DSR.\r\n" ;
static UCHAR Message07 [ ]   = "Wlaczono linie CTS.\r\n" ;
static UCHAR Message08 [ ]   = "Wylaczono linie CTS.\r\n" ;

int main ( void )
{
  UCHAR ModemStatus ;
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART1HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  for ( ; ; )
  {
    if ( UART1DataPresent ( ) )
      ProcessChar ( UART1GetData ( ) ) ;
    ModemStatus = GetInputModemLine ( ) ;
    if ( LastDCDModemLine )
    {
      if ( ! ( ModemStatus & DCDControlUARTCode ) )
      {
        UART1SendString ( Message02 ) ;
        LastDCDModemLine = 0 ;
      } /* if */ ; 
    } /* if ... */
    else
    {
      if ( ModemStatus & DCDControlUARTCode )
      {
        UART1SendString ( Message01 ) ;
        LastDCDModemLine = 1 ;
      } /* if */ ; 
    } /* if ... else */ ;
    if ( LastRIModemLine )
    {
      if ( ! ( ModemStatus & RIControlUARTCode ) )
      {
        UART1SendString ( Message04 ) ;
        LastRIModemLine = 0 ;
      } /* if */ ; 
    } /* if ... */
    else
    {
      if ( ModemStatus & RIControlUARTCode )
      {
        UART1SendString ( Message03 ) ;
        LastRIModemLine = 1 ;
      } /* if */ ; 
    } /* if ... else */ ;
    if ( LastDSRModemLine )
    {
      if ( ! ( ModemStatus & DSRControlUARTCode ) )
      {
        UART1SendString ( Message06 ) ;
        LastDSRModemLine = 0 ;
      } /* if */ ; 
    } /* if ... */
    else
    {
      if ( ModemStatus & DSRControlUARTCode )
      {
        UART1SendString ( Message05 ) ;
        LastDSRModemLine = 1 ;
      } /* if */ ; 
    } /* if ... else */ ;
    if ( LastCTSModemLine )
    {
      if ( ! ( ModemStatus & CTSControlUARTCode ) )
      {
        UART1SendString ( Message08 ) ;
        LastCTSModemLine = 0 ;
      } /* if */ ; 
    } /* if ... */
    else
    {
      if ( ModemStatus & CTSControlUARTCode )
      {
        UART1SendString ( Message07 ) ;
        LastCTSModemLine = 1 ;
      } /* if */ ; 
    } /* if ... else */ ;
  } /* loop */ ;
} /* main */

