/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                       Copyright (C) A. P. H.                           **
**                           BIALYSTOK 2019                               **
**                                                                        **
**                        MODULE UART2 example                            **
**                                                                        **
**                     Mikrokontroler ARM: LPC2366                        **
**                                                                        **
**                     Wersja z dnia : 2019.01.20                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart3.h"

#define IRQCallCntLimit                 500
#define BufferSize                      64

static ULONG IRQCallCounter ;
static USHORT Index ;
static UCHAR Buffer [ BufferSize ] ;

static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  Index = 0 ;
} /* SoftwareInit */


static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART3InitEnvir ( ) ;
} /* HardwareInit */


UCHAR Hello1 [ ]      = "\r\nWitaj koles przy kompie\r\n\r\nNapisz cos\r\n" ;
UCHAR Hello2 [ ]      = "\r\nNapisales: " ;


static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART3SendString ( Hello1 ) ;
} /* HelloMessage */


void ProcessChar ( UCHAR Ch )
{
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  if ( Ch < ' ' )
  {
    if ( Index )
    {
      for ( Loop = 0 ; Loop < Index ; Loop ++ )
        UART3Send ( Buffer [ Loop ] ) ;
      UART3Send ( 0x0D ) ;
      UART3Send ( 0x0A ) ;
    } /* if */ ;
    Index = 0 ;
  } /* if ... */
  else
  {
    if ( Index < BufferSize )
    {
      Buffer [ Index ] = Ch ;
    } /* IF */ ;
    Index ++ ;
  } /* if ... else */ ;
} /* ProcessChar */


int main ( void )
{
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART3HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  for ( ; ; )
  {
    if ( UART3DataPresent ( ) )
      ProcessChar ( UART3GetData ( ) ) ;
  } /* loop */ ;
} /* main */



